/* DuduSDL
 * dudusdl.c
 *
 * Copyright (C) 2000 Nicolas Peninguy and Colin Leroy
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "dudusdl.h"

int LoadTXT(char *fichier, int num, int w, int h)
{
  SDL_Surface *s;
  int i;
  SDL_Rect rect;
  
  assert( ((num >= 0) & (num <= NB_TXT)) );
  
  rect.y = 1;
  rect.w = w;
  rect.h = h;
  
  if (! (s = IMG_Load(fichier))) {
    printf("Can't load file\n");
    return 1;
  }
  
  for(i = 0; i < 39; i++) {
    if (! (polices[num].character[i] =
	   SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 8, 0, 0, 0, 0)))
      return 1;
    
    SDL_SetPalette(polices[num].character[i], SDL_LOGPAL|SDL_PHYSPAL,
		   s->format->palette->colors, 0, s->format->palette->ncolors);
    
    rect.x = ((w+1) * i) + 1;
    
    if (SDL_BlitSurface(s, &rect, polices[num].character[i], NULL) != 0)
      return 1;
    
    SDL_SetColorKey(polices[num].character[i], SDL_SRCCOLORKEY, SDL_MapRGB(screen->format,0,0,0));
  }
  
  polices[num].width = w;
  return 0;
}

void PutTXTC(int num, char *string, int x, int y)
{
  int i=0;
  int c;
  SDL_Rect rect;

  rect.x = x;
  rect.y = y;
  rect.w = 0;
  rect.h = 0;
  
  while ((c = string[i]) != '\0') {
    if (isdigit(c))
      SDL_BlitSurface(polices[num].character[c - '0' + 26], NULL, screen, &rect);
    else if (isalpha(c))
      SDL_BlitSurface(polices[num].character[toupper(c) - 'A'], NULL, screen, &rect);
    else if (c == '.')
      SDL_BlitSurface(polices[num].character[36], NULL, screen, &rect);
    else if (c == ',')
      SDL_BlitSurface(polices[num].character[37], NULL, screen, &rect);
    else if (c == '?')
      SDL_BlitSurface(polices[num].character[37], NULL, screen, &rect);
    i++;
    rect.x += polices[num].width;
  }
}

void PutTXT(int num, char *string, int num2, int x, int y)
{
  PutTXTC(num, string, x, y);
}

void pixel(SDL_Surface *dest, int x, int y, int color)
{
  Uint8 *bits;

  /************ Precondition **************/
  // Cette routine ne fonctionne qu'en mode 8 bits.
  assert((dest->format->BytesPerPixel = 1));
  // Ne pas sortir de la zone de dessin !
  assert( ((x>=0) && (y>=0) && (x<dest->w) && (y<dest->h)) );
  // Couleur entre 0 et 255
  assert( ((color>=0) && (color<256)) );
  /****************************************/

  if (SDL_MUSTLOCK(dest))
    if (SDL_LockSurface(dest) < 0)
      return;
  
  bits = ((Uint8 *)dest->pixels) + y*dest->pitch + x;
  *bits = (Uint8)color;
  
  if (SDL_MUSTLOCK(dest))
    SDL_UnlockSurface(dest);

  return;
} // pixel

void line(SDL_Surface *dest, int x1, int y1, int x2, int y2, int color)
{
  int dx, dy;
  int linc, sinc;
  int dl, ds;
  int err;
  int i;
  Uint8 *bits;
  
  /*********** Preconditions ***************/
  // Cette routine ne fonctionne qu'en 8 bits.
  assert((dest->format->BytesPerPixel = 1));
  // Ne pas sortir de la zone de dessin !
  assert( ((x1>=0) && (x2>=0) && (y1>=0) && (y2>=0)) );
  assert( ((x1<dest->w) && (x2<dest->w) && (y1<dest->h) && (y2<dest->h)) );
  // Couleur entre 0 et 255
  assert( ((color>=0) && (color<256)) );
  /*****************************************/

  if (SDL_MUSTLOCK(dest))
    if (SDL_LockSurface(dest) < 0)
      return;

  dx = x2 - x1;
  dy = y2 - y1;
  
  if (abs(dx) > abs(dy)) {
    dl = abs(dx);
    ds = abs(dy);
    linc = (dx > 0) ? 1 : -1;
    sinc = (dy > 0) ? dest->pitch : -dest->pitch;
  } else {
    dl = abs(dy);
    ds = abs(dx);
    sinc = (dx > 0) ? 1 : -1;
    linc = (dy > 0) ? dest->pitch : -dest->pitch;
  }
  
  bits = ((Uint8 *)dest->pixels) + y1*dest->pitch + x1;
  
  err = dl/2;

  for (i=0; i<=dl; i++) {
    *bits = (Uint8)color;
    bits += linc;
    err += ds;
    if (err > dl) {
      bits += sinc;
      err -= dl;
    }
  }
  
  if (SDL_MUSTLOCK(dest))
    SDL_UnlockSurface(dest);
} // line

